function [plotV,imS,reS,VCR,OmegaCR,p,FlutterMode]=AerodynamicStability(MM,CC,KK,omegaomega,I,rho,B,varargin)
%% Description
% This routine calcualates the velocity dependent eigenvalue problem for a
% bridge subjected to wind loading
%% Define input parameters
p=inputParser;
addParameter(p,'maxVit',100,@isnumeric);                  %Maximum number of velocity iterations
addParameter(p,'maxV',300,@isnumeric);                    %Maximum mean wind velocity
addParameter(p,'maxfit',50,@isnumeric);                   %Maximum number of frequency iterations
addParameter(p,'dV',20,@isnumeric);                       %delta velocity
addParameter(p,'V',40,@isnumeric);                        %initial velocity
addParameter(p,'omegamin',0,@isnumeric);                  %minimum circular frequency of interest
addParameter(p,'omegamax',Inf,@isnumeric);                %maximum circular frequency of interest
addParameter(p,'RAD',[],@isnumeric);                      %Random variables related to aerodynamic derivatives
addParameter(p,'d',[],@isnumeric);                        %Used for Rational Functions
addParameter(p,'CriteriaRe',0.001,@isnumeric);             %Convergence criteria for the real part of the eigenvalue
addParameter(p,'CriteriadV',0.05,@isnumeric);             %Convergence criteria for the velocity increment
addParameter(p,'PlotResults','No',@ischar);               %Plot the real and imaginary part of the eigenvalues

addParameter(p,'PlotAD','No',@ischar);                    %Plot the aerodynamic derivatives
addParameter(p,'ADType','Theodorsen',@ischar);            %Aerodynamic derivatives type

parse(p,varargin{:})
dV=p.Results.dV;
V=p.Results.V;
%% Perform the calculations
VCR=NaN;    % Nan is returned if the calculations does not converge                    
OmegaCR=NaN;    % Nan is returned if the calculations does not converge 
FlutterMode=NaN;
tmp=find((omegaomega<p.Results.omegamax) & (omegaomega>p.Results.omegamin));
first_n=tmp(1,1);
last_n=tmp(length(tmp),1);
last_n=min([last_n,size(MM,1)]);
antfit=0;
stopfit=0;
antVit=0;
stopVit=0;
pos=1;
ResMat=zeros(2,2*size(MM,1));
plotV=zeros(1,p.Results.maxVit);
imS=zeros(2*size(MM,1),p.Results.maxVit);
reS=zeros(2*size(MM,1),p.Results.maxVit);
%% Beregning av stabilitetsgrensen
while antVit<p.Results.maxVit && stopVit==0 && V<p.Results.maxV
    %% Iterasjon p frekvenser
    v=first_n-1;
    while v<last_n
        v=v+1;
        omegacr=omegaomega(v,1);
        while antfit<p.Results.maxfit&&stopfit==0
            [KaeKae,CaeCae]=Self_exited_load(V,MM,omegacr,I,rho,B,p);
            KKT=KK-KaeKae;CCT=CC-CaeCae;
            [~,e]=polyeig(KKT,CCT,MM);
            [sortertefrek, posid]=sort(imag(e),'ascend');
            sortertefrek=sortertefrek(size(posid,1)/2+1:size(posid,1));
            domega=omegacr-sortertefrek(v,1);
            disp('************************')
            omegacr=sortertefrek(v,1);
            if abs(domega)<0.0001
                stopfit=1;
                
            elseif omegacr<=0
                stopfit=1;
            end
            antfit=antfit+1;
            if antfit==p.Results.maxfit
                beep
                disp('*****************************************************')
                disp('The frequency iterations has not converged')
                disp(['Mean wind velocity V=' num2str(V) ',frequency \omega=' num2str(omegacr) ])
                disp('*****************************************************')
                stopfit=1;
            end
        end
        stopfit=0;antfit=0;
        posres=find(omegacr==abs(imag(e)));
        if size(posres,1)>2 % More then two eigenvalues with the same imaginary part
            ResMat(1,2*v-1:2*v-1+size(posres,1)-1)=imag(e(posres));
            ResMat(2,2*v-1:2*v-1+size(posres,1)-1)=real(e(posres));
            v=v+size(posres,1)/2-1; %Skip the eigenvalues with the same imaginary part since these have converged
        else
            ResMat(1,2*v-1:2*v)=imag(e(posres));
            ResMat(2,2*v-1:2*v)=real(e(posres));
        end
    end
    %% Oppsamling av data
    [stabind, pos_stabind]=max(ResMat(2,:));
    antVit=antVit+1;
    if stabind>0 && ne(ResMat(1,pos_stabind),0)
        V=V-0.5*dV;
        dV=0.5*dV;
    else
        plotV(1,pos)=V;
        imS(:,pos)=ResMat(1,:)';
        reS(:,pos)=ResMat(2,:)';
        pos=pos+1;
        V=V+dV;
    end
    if abs(stabind)<p.Results.CriteriaRe && dV<p.Results.CriteriadV
        stopVit=1;
        [Demping, posCR]=max(ResMat(2,:));
        OmegaCR=ResMat(1,posCR);
        VCR=V;
        % Obtain flutter mode
        [KaeKae,CaeCae]=Self_exited_load(VCR,MM,OmegaCR,I,rho,B,p);
        KKT=KK-KaeKae;CCT=CC-CaeCae;
        [vec,e]=polyeig(KKT,CCT,MM);
%         FlutterMode=vec(:,(abs(real(e./abs(e)))<0.1/1000));
%         FlutterMode=FlutterMode(:,1);
%         FlutterMode=FlutterMode/max(abs(FlutterMode));
        
    end
    ResMat=zeros(2,2*size(MM,1));
    plotV=nonzeros(plotV)';
    imS=imS(:,1:size(plotV,2));
    reS=reS(:,1:size(plotV,2));
end
plotV=nonzeros(plotV);
% imS=imS(2*first_n:2*last_n,1:size(plotV,1));
% reS=reS(2*first_n:2*last_n,1:size(plotV,1));

%% Plot the results if selected
if strcmp(p.Results.PlotResults,'No')==0
    figure
    subplot(2,1,1)
    hold all
    plot(plotV,imS')
    ylabel('Im(S)')
    ylim([0 3])
    subplot(2,1,2)
    hold all
    plot(plotV,reS')
    ylim([-0.5 0])
    ylabel('Re(S)')
    xlabel('V (m/s)')
    drawnow
    UsedVred=[];
    for n=1:length(plotV)
        UsedVred=[UsedVred plotV(n,1)./(B*abs(imS(:,n)))'];
    end
    [AD]=AerodynamicDerivatives(UsedVred,'RAD',p.Results.RAD,'d',p.Results.d,'ADType',p.Results.ADType);
    plotAerodynamicDerivatives(UsedVred,AD,'DataType','Exp')
    
    
end
function [KaeKae,CaeCae]=Self_exited_load(V,MM,omegacr,I,rho,B,p)
% Initielle beregninger
if omegacr==0
    omegacr=1.0e-20;
end
vhatt=V/(B*omegacr);
aerovektor=AerodynamicDerivatives(vhatt,'PlotAD',p.Results.PlotAD,'RAD',p.Results.RAD,'d',p.Results.d,'ADType',p.Results.ADType);
kaekaeorg=(omegacr)^2*rho*B^2/2*[aerovektor(4,1) aerovektor(6,1) B*aerovektor(3,1);aerovektor(12,1) aerovektor(10,1) B*aerovektor(9,1);B*aerovektor(18,1) B*aerovektor(16,1) B^2*aerovektor(15,1)];
caecaeorg=omegacr*rho*B^2/2*[aerovektor(1,1) aerovektor(5,1) B*aerovektor(2,1);aerovektor(11,1) aerovektor(7,1) B*aerovektor(8,1); B*aerovektor(17,1) B*aerovektor(13,1) B^2*aerovektor(14,1)];
KaeKae=zeros(size(MM,1),size(MM,1));
CaeCae=zeros(size(MM,1),size(MM,1));
for i=1:size(MM,1)
    for j=1:size(MM,1)
        KaeKae(i,j)=sum(sum(I(:,:,i,j).*kaekaeorg));
        CaeCae(i,j)=sum(sum(I(:,:,i,j).*caecaeorg));
    end
end
